<?php

/**
 * @file
 * Serves administration pages of unique_comments.
 */

/**
 * Admin main page.
 */

/**
 * Menu callback: Shows the unique comments settings.
 */
function unique_comments_admin_form() {

  $content_types = node_get_types();
  foreach ($content_types as $ct) {
    $types[$ct->type] = check_plain($ct->name);
  }

  $form['unique_comments_status'] = array(
    '#type' => 'radios',
    '#title' => t('Unique Comments Status'),
    '#default_value' => variable_get('unique_comments_status', 'None'),
    '#options' => drupal_map_assoc(array('None', 'Per Node', 'Site Wide')),
    '#description' => t('Select to ensure originality either: not at all, per node or sitewide!'),
  );

  $form['unique_comments_content_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Content Types'),
    '#description' => t('Select which content types Unique Comments will work on. If no content types are ticked, it is equivalent to selecting "None" in the above section.'),
    '#options' => $types,
    '#default_value' => variable_get('unique_comments_content_types', array()),
  );

  return system_settings_form($form);
}

/**
 * Callback for the node form to display all nodes exempted from Unique Comments
 */
function unique_comments_node_list() {
  $header = array(
    array(
      'data' => t('Added'),
      'field' => 'added',
    ),
    array(
      'data' => t('Node ID'),
      'field' => 'nid',
    ),
    array(
      'data' => t('Added by UID'),
      'field' => 'uid',
    ),
    array(
      'data' => t('Operations'),
    ),
  );

  $rows = array();

  $query = db_query("SELECT uc.ucid, uc.nid, uc.uid, uc.added FROM {unique_comments} uc" . tablesort_sql($header));

  while ($data = db_fetch_object($query)) {
    $rows[] = array(
      'data' => array(
        format_date($data->added),
        l($data->nid, 'node/' . $data->nid),
        l($data->uid, 'user/' . $data->uid),
        l(t('delete'), 'admin/content/unique_comments/delete/' . $data->ucid)
      ),
    );
  }
  if (empty($rows)) {
    $rows[] = array(
      array(
        'data' => t('There are currently no nodes exempt from Unique Comments.'),
        'colspan' => '4',
        'align' => 'center',
        'class' => 'message',
      ),
    );
  }

  return theme('table', $header, $rows);
}

/**
 * Callback to delete a node from being exempt from Unique Comments
 */
function unique_comments_node_exemption_delete(&$form, $ucid) {

  $result = db_result(db_query('SELECT nid from {unique_comments} where ucid = %d', array($ucid)));

  $form = array();
  $form['ucid'] = array(
    '#type' => 'value',
    '#value' => $ucid,
  );
  // Following line stops a weird error I kept getting upon submitting.
  // Referred to form.inc line 1041 and how submitted wasn't an index.
  // This could be related to setting $form = array(); but that itself
  // was to stop a WSOD error.
  $form['submitted'] = FALSE;
  if ($result) {
    return confirm_form($form,
                  t('Are you sure you want to delete this exemption?'),
                  'admin/content/unique_comments/nodes',
                  t('Deleting this will cause node @nid to be affected by Unique Comments again.', array('@nid' => $result)),
                  t('Delete'),
                  t('Cancel'));
  }
  else {
    drupal_set_message(t('Invalid parameters'), 'warning');
    drupal_goto('admin/content/unique_comments/nodes');
  }

  return $output;

}

/**
 * Submit function for deleting a nid from the Unique Comments table
 */
function unique_comments_node_exemption_delete_submit($form, &$form_state) {
  db_query("DELETE FROM {unique_comments} WHERE ucid = %d", $form_state['values']['ucid']);

  drupal_set_message(t('The exemption has been deleted.'));
  $form_state['redirect'] = 'admin/content/unique_comments/nodes';
}

/**
 * Form callback for adding an exemption for a node from Unique Comments
 */
function unique_comments_node_exemption_add() {
  $form['unique_comments_node_exemption_add'] = array(
    '#type' => 'textfield',
    '#title' => t('Add Exemption'),
    '#description' => t('Add in a Node ID here to create a new exemption from Unique Comments'),
    '#size' => 10,
    '#element_validate' => array('_unique_comments_node_exemption_add_field_validate'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
  );
  return $form;
}

/**
 * Form validation callback for adding node to exemptions.
 * This is done to ensure the NID is numeric and that the node exists.
 */
function _unique_comments_node_exemption_add_field_validate($element, &$form_state) {
  if (!is_numeric($element['#value'])) {
    form_error($element, t('Node IDs must be numeric!'));
  }
  $node = node_load($element['#value']);
  if (empty($node)) {
    form_error($element, t("The node doesn't currently exist, please use a valid NID"));
  }
  else {
    $type = node_get_types('type', $node);
    if (!in_array($node->type, variable_get('unique_comments_content_types', array(TRUE)), TRUE)) {
      form_error($element, t('The node type "@type" is not currently allowed by Unique Comments', array('@type' => $type->name)));
    }
    if (unique_comments_node_is_disabled($element['#value'])) {
      form_error($element, t('The Node ID selected has already been exempted.'));
    }
  }
}

/**
 * Form submit function to insert the record into the Unique Comments db.
 */
function unique_comments_node_exemption_add_submit($form, &$form_state) {
  global $user;
  db_query("INSERT INTO {unique_comments}
            (nid, uid, added)
            VALUES (%d, %d, %d)",
            array($form_state['values']['unique_comments_node_exemption_add'], $user->uid, time())
          );
  drupal_set_message(t('Node ID @nid added to the exemption list', array('@nid' => $form_state['values']['unique_comments_node_exemption_add'])));
}
